#!/bin/sh
. /usr/share/libubox/jshn.sh

SECTION=$1

ENABLE=$(uci -q get ping_reboot."$SECTION".enable)
TIMEOUT=$(uci -q get ping_reboot."$SECTION".time_out)
RETRIES=$(uci -q get ping_reboot."$SECTION".retry)
HOST=$(uci -q get ping_reboot."$SECTION".host)
ACTION=$(uci -q get ping_reboot."$SECTION".action)
CURRENT_TRY=$(uci -q get ping_reboot."$SECTION".current_try)
PACKET_SIZE=$(uci -q get ping_reboot."$SECTION".packet_size)
MODEM_EXEC=$(uci -q get ping_reboot."$SECTION".modem)
TYPE=$(uci -q get ping_reboot."$SECTION".type)

FILE="/tmp/wget_check_file"

debug() {
	logger -t ping_reboot.sh "$1"
}

get_modem() {
    local modem modems id builtin primary
    local primary_modem=""
    local builtin_modem=""
    json_init
    json_load "$(cat /etc/board.json)"
    json_get_keys modems modems
    json_select modems

    for modem in $modems; do
        json_select "$modem"
        json_get_vars builtin primary id
        if [ "$builtin" -a "$id" ]; then
            builtin_modem=$id
        fi
        if [ "$primary" -a "$id" ]; then
            primary_modem=$id
            break
        fi
        json_select ..
    done

    if [ "$primary_modem" = "" ]; then
        if [ "$builtin_modem" = "" ]; then
            json_load "$(/bin/ubus call gsmd get_modems)"
            json_get_keys modems modems
            json_select modems

            for modem in $modems; do
                json_select "$modem"
                json_get_vars id
                primary_modem=$id
                break
            done
        else
            primary_modem=$builtin_modem
        fi
    fi
    uci -q set ping_reboot."$SECTION".modem="$primary_modem"
    uci commit ping_reboot
    MODEM_EXEC=$primary_modem
}

exec_action() {
    uci -q set ping_reboot."$SECTION".current_try=0
	uci commit ping_reboot
    case "$ACTION" in
        "1")
            debug "Rebooting router after $RETRIES unsuccessful tries"
            reboot 
            ;;
        "2")
            debug "Restarting modem after $RETRIES unsuccessful tries"
            /bin/ubus call gsmd reinit_modems "{\"id\":\"$MODEM_EXEC\"}"
            ;;
        "3")
            ;;
	esac
}

if [ "$ENABLE" == "1" ]; then
    if [ "$MODEM_EXEC" = "" ]; then
        get_modem
    fi
	CURRENT_TRY=$((CURRENT_TRY + 1))
	uci -q set ping_reboot."$SECTION".current_try=$CURRENT_TRY
	uci commit ping_reboot

    if [ "$TYPE" = "wget" ]; then
	    wget -q -T $TIMEOUT $HOST -O $FILE &> /dev/null
	    if [ ! -s $FILE ];then
            debug "Can't wget URL."
            if [ $CURRENT_TRY -ge $RETRIES ]; then
                exec_action "-g"
            else
                debug "Will be retrying wget"
            fi
        else
            uci -q set ping_reboot."$SECTION".current_try=0
            uci commit ping_reboot
            debug "Wget URL successful."
        fi
        rm $FILE &> /dev/null
	else
	    result=$(ping -W $TIMEOUT -s $PACKET_SIZE -q -c 1 $HOST &> /dev/null; echo $?)
	    if [ "$result" != "0" ]; then
	        debug "Can't ping URL."
	        if [ $CURRENT_TRY -ge $RETRIES ]; then
	            exec_action "-p"
	        else
	            debug "Will be retrying ping"
	        fi
	    else
	        uci -q set ping_reboot."$SECTION".current_try=0
            uci commit ping_reboot
            debug "Ping URL successful."
        fi
    fi
fi
